/*
 * Linux VeeJay
 *
 * Copyright(C)2002 Niels Elburg < nelburg@sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License , or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 , USA.
 */

#ifndef SAMPLEADM_H
#define SAMPLEADM_H
#include <stdint.h>
#include "hash.h"
#include "lav_io.h"
#include "vj-effman.h"
#include "vj-ffmpeg.h"
#include "vj-global.h"
/*
 * COMPAT using xml-config --cflags to get the include path this will
 * work with both 
 *
 *
 * sampleadm is responsible for storage/retrieving sample information.
 * it is one of the key components in veejay.
 */
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#define SAMPLE_MAX_RENDER    10 /* 10 at most */
#define SAMPLE_MAX_SAMPLES 4096	/* 4096 samples at most */

#define SAMPLE_MAX_PARAMETERS 10	/* 10 parameters per effect at most */
#define SAMPLE_ARG1 0
#define SAMPLE_ARG2 1
#define SAMPLE_ARG3 2
#define SAMPLE_ARG4 3
#define SAMPLE_ARG5 4
#define SAMPLE_ARG6 5
#define SAMPLE_ARG7 6
#define SAMPLE_ARG8 7
#define SAMPLE_ARG9 8
#define SAMPLE_ARG10 9

#define SAMPLE_FREEZE_NONE 0
#define SAMPLE_FREEZE_PAUSE 1
#define SAMPLE_FREEZE_BLACK 2

#define SAMPLE_RENDER_START 1
#define SAMPLE_RENDER_STOP 0
#define SAMPLE_MAX_EFFECTS 10
#define XMLTAG_RENDER_ENTRY "render_entry"
#define XMLTAG_SAMPLES    "veejay_samples"
#define XMLTAG_SAMPLE     "sample"
#define XMLTAG_SAMPLEID   "sampleid"
#define XMLTAG_SAMPLEDESCR "description"
#define XMLTAG_FIRSTFRAME "startframe"
#define XMLTAG_LASTFRAME  "endframe"
#define XMLTAG_EFFECTS    "effects"
#define XMLTAG_VOL	  "volume"
#define XMLTAG_EFFECT     "effect"
#define XMLTAG_EFFECTID   "effectid"
#define XMLTAG_ARGUMENTS  "arguments"
#define XMLTAG_ARGUMENT   "argument"
#define XMLTAG_EFFECTSOURCE "source"
#define XMLTAG_EFFECTCHANNEL "channel"
#define XMLTAG_EFFECTTRIMMER "trimmer"
#define XMLTAG_EFFECTOFFSET "offset"
#define XMLTAG_EFFECTACTIVE "active"
#define XMLTAG_EFFECTAUDIOFLAG "use_audio"
#define XMLTAG_EFFECTAUDIOVOLUME "chain_volume"
#define XMLTAG_RANDOMIZER "randomizer"
#define XMLTAG_RANDARGNR  "random_arg"
#define XMLTAG_RANDDELAY  "random_delay"
#define XMLTAG_RANDMIN    "random_min"
#define XMLTAG_RANDMAX    "random_max"
#define XMLTAG_SPEED      "speed"
#define XMLTAG_FRAMEDUP   "frameduplicator"
#define XMLTAG_LOOPTYPE   "looptype"
#define XMLTAG_MAXLOOPS   "maxloops"
#define XMLTAG_NEXTSAMPLE "nextsample"
#define XMLTAG_DEPTH	  "depth"
#define XMLTAG_PLAYMODE   "playmode"
#define XMLTAG_VOLUME	  "volume"
#define XMLTAG_SUBAUDIO	  "subaudio"
#define XMLTAG_MARKERSTART "markerstart"
#define XMLTAG_MARKEREND   "markerend"
#define XMLTAG_EFFECTPOS   "position"


enum {
    SAMPLE_LOAD = 0,
    SAMPLE_RUN = 1,
    SAMPLE_PEEK = 2,
};

typedef struct sample_eff_t {
    int effect_id;		/* effect ID */
    int e_flag;
    /* enable / disable */
    int arg[SAMPLE_MAX_PARAMETERS];	/* array of arguments */
    int frame_offset;
    int frame_trimmer;		/* sub frame scratcher */
    /* audio settings */
    int a_flag;			/* audio enabled/disabled */
    int volume;			/* volume of 0-100 of audio */
    int source_type;		/* source type to mix with */
    int channel;		/* secundary source id */
    int is_rendering;		/* is rendering */
} sample_eff_chain;

typedef struct rand_eff_t {
    int *delay;			/* frame count before trigger */
    int *limits[2];		/* min, max values to randomize */
    int *arg_nr;		/* which effect parameter */
} rand_eff_chain;

typedef struct sample_info_t {
    int sample_id;		/* identifies a unique sample */
    sample_eff_chain *effect_chain[SAMPLE_MAX_EFFECTS];	/* effect chain */
    rand_eff_chain *random_chain[SAMPLE_MAX_EFFECTS];	/* randomizer chain */
    long first_frame[SAMPLE_MAX_RENDER];		/* start of sample */
    long last_frame[SAMPLE_MAX_RENDER];		/* end of sample */
    char descr[150];
    int speed;			/* playback speed */
    int looptype;		/* pingpong or loop */
    int max_loops;		/* max looops before going to */
    int max_loops2;		/* count remaining loops */
    int next_sample_id;		/* the next sample */
    int depth;			/* sample effect chain render depth */
    int source;			/* source or tag */
    int channel;		/* which channel (which tag) */
    int playmode;		/* various playmodes */
    int playmode_frame;
    int sub_audio;		/* mix underlying sample yes or no */
    int audio_volume;		/* volume setting of this sample */
    int marker_start;
    int marker_end;
    int dup;			/* frame duplicator */
    int active_render_entry;
    int loop_dec;
    int loop_periods;
    int freeze_mode;
    int freeze_nframes;
    int freeze_pframes;
    int freeze_val_nframes;
    int freeze_val_pframes;
    int marker_speed;
    int fader_active;
    int fader_direction;
    float fader_val;
    float fader_inc;
    int encoder_active;
    char *encoder_destination;
    int encoder_format;
    lav_file_t *encoder_file;
    long encoder_duration; /* in seconds */
    long encoder_num_frames;
    long encoder_succes_frames;
    int encoder_width;
    int encoder_height;
    int encoder_max_size;
    uint8_t *encoder_buf;
    int auto_switch;	
} sample_info;

#define SAMPLE_YUV420_BUFSIZE 16
#define SAMPLE_MAX_DEPTH 4
#define SAMPLE_DEC_BIBBER 1
#define SAMPLE_DEC_FREEZE 2


int sample_size();
int sample_verify();
void sample_init();
int sample_update(sample_info *sample, int s1);
int sample_readFromFile(char *);
int sample_writeToFile(char *);

int sample_set_state(int new_state);
int sample_get_state();

sample_info *sample_skeleton_new(long startFrame, long endFrame);
sample_info *sample_get(int sample_id);
int sample_store(sample_info * skel);
int sample_is_deleted(int s1);
int sample_exists(int sample_id);
int sample_del(int sample_id);
void sample_del_all();
int sample_get_startFrame(int sample_id);
int sample_get_endFrame(int sample_id);
int sample_set_marker_start(int sample_id, int marker);
int sample_set_marker_end(int sample_id, int marker);
int sample_set_startframe(int s1, long frame_num);
int sample_set_endframe(int s1, long frame_num);
int sample_set_marker(int s1, int start, int end);

int sample_get_playmode(int s1);
int sample_set_playmode(int s1, int playmode);
int sample_get_loops(int s1);
int sample_get_loops2(int s1);
int sample_get_next(int s1);
int sample_get_depth(int s1);
int sample_set_depth(int s1, int n);
int sample_set_speed(int s1, int speed);
int sample_set_framedup(int s1, int n);
int sample_get_framedup(int s1);
int sample_marker_clear(int sample_id); 
int sample_set_looptype(int s1, int looptype);
int sample_get_speed(int s1);
int sample_get_looptype(int s1);
int sample_set_loops(int s1, int nr_of_loops);
int sample_set_loops2(int s1, int nr);
int sample_set_next(int s1, int next_sample_id);

int sample_get_chain_source(int sample_id, int position);
int sample_set_chain_source(int sample_id, int position, int source);
int sample_get_sub_audio(int s1);
int sample_set_sub_audio(int s1, int audio);
int sample_get_audio_volume(int s1);
int sample_set_audio_volume(int s1, int volume);
int sample_copy(int s1);
int sample_get_effect(int s1, int position);
/* get effect any, even if effect is disabled (required for informational purposes)*/
int sample_get_effect_any(int s1, int position);
int sample_get_offset(int s1, int position);

/* trimmer is usefull for underlying samples in the effect chain.
   you can manual adjust the video/audio sync of the underlying sample */
int sample_get_trimmer(int s1, int position);
int sample_set_trimmer(int s1, int position, int trimmer);
int sample_get_short_info(int sample_id, int *, int *, int *, int *) ;
int sample_get_chain_volume(int s1, int position);

/* set volume of audio data coming to the chain */
int sample_set_chain_volume(int s1, int position, int volume);

/* whether to mix underlying sample's audio */
int sample_get_chain_audio(int s1, int position);
int sample_has_extra_frame(int s1, int position);
/* mix the audio from entry from the effect chain, if any */
int sample_set_chain_audio(int s1, int position, int flag);

int sample_set_chain_status(int s1, int position, int status);
int sample_get_chain_status(int s1, int position);

int sample_set_offset(int s1, int position, int frame_offset);
int sample_get_effect_arg(int s1, int position, int argnr);
int sample_set_effect_arg(int s1, int position, int argnr, int value);

int sample_get_all_effect_arg(int s1, int position, int *args,
			      int arg_len);
int sample_chain_set_arg_as_random(int s1, int position, int p);
int sample_chain_get_arg_to_randomize(int s1, int position, int p);
int sample_chain_get_rand_delay(int s1, int position, int p);
int sample_chain_set_rand_delay(int s1, int position, int p,
				int rand_delay);
int sample_chain_set_rand_range(int s1, int position, int arg, int min,
				int max);
int sample_chain_get_min_rand(int s1, int position, int p);
int sample_chain_get_max_rand(int s1, int position, int p);
int sample_chain_remove(int s1, int position);
int sample_chain_clear(int s1);
int sample_chain_size(int s1);
int sample_chain_get_free_entry(int s1);
int sample_chain_add(int s1, int c, int effect_nr);

/* channel depends on source , it select a channel of a certain video source */
int sample_get_chain_channel(int s1, int position);
int sample_set_chain_channel(int s1, int position, int channel);

//int sample_chain_replace(int s1, int position, int effect_id);

int sample_chain_sprint_status(int s1, int entry, int changed, int r_changed, char *str,
			       int frame);

int sample_set_render_entry(int s1, int entry);
int sample_get_render_entry(int s1);

int sample_set_description(int sample_id, char *description);
int sample_get_description(int sample_id, char *description);

int sample_entry_is_rendering(int sample_id, int entry);
int sample_entry_set_is_rendering(int sample_id, int entry, int value);
int sample_get_loop_dec(int s1);
int sample_set_loop_dec(int s1, int active, int periods);
int sample_apply_loop_dec(int s1, double fps); 

int sample_set_freeze_mode(int s1, int mode, int nframes, int pframes);
int sample_get_freeze_nframes(int s1);
int sample_get_freeze_pframes(int s1);
int sample_get_freeze_mode(int s1);
int sample_set_freeze_pframes(int s1 , int nframes);
int sample_set_freeze_nframes(int s1,  int nframes);
int sample_reset_freeze(int s1);

int sample_set_fader_active(int t1, int nframes, int direction);
int sample_set_fader_val(int t1, float val);
int sample_get_fader_active(int t1);
float sample_get_fader_val(int t1);
float sample_get_fader_inc(int t1);
int sample_get_fader_direction(int s1);
int sample_reset_fader(int t1);


void CreateSample(xmlNodePtr node, sample_info * sample);
void CreateEffects(xmlNodePtr node, sample_eff_chain ** effects);
void CreateEffect(xmlNodePtr node, sample_eff_chain * effect, int pos);
void CreateArguments(xmlNodePtr node, int *arg, int argcount);
void ParseSample(xmlDocPtr doc, xmlNodePtr cur, sample_info * skel);
void ParseEffects(xmlDocPtr doc, xmlNodePtr cur, sample_info * skel);
void ParseEffect(xmlDocPtr doc, xmlNodePtr cur, int dst_sample);
void ParseArguments(xmlDocPtr doc, xmlNodePtr cur, int *arg);
unsigned char *UTF8toLAT1(unsigned char *in);


#endif
